#ifndef _checkServiceH
#define _checkServiceH
	
#include "windows.h"
#include <stdio.h>

// Valid status codes for a service, returned to the client
#define	SERV_SERVICE_DOES_NOT_EXIST		1
#define	SERV_SERVICE_NOT_ACTIVE			2
#define	SERV_SERVICE_REQUEST_TIMEOUT	3
#define	SERV_CHECK_ERROR				4
#define	SERV_PAUSED						5
#define SERV_RUNNING					6

//
//  FUNCTION: DWORD CheckService(char *servName, DWORD &status)
//
//  PURPOSE: Check the current status of the specified service 
//
//  PARAMETERS:
//    servName - the display name of the service (case sensitive!)
//    stat - the status of the service. 
//			Following states are possible:
//				SERVICE_STOPPED				The service is not running. 
//				SERVICE_START_PENDING		The service is starting. 
//				SERVICE_STOP_PENDING		The service is stopping. 
//				SERVICE_RUNNING				The service is running. 
//				SERVICE_CONTINUE_PENDING	The service continue is pending. 
//				SERVICE_PAUSE_PENDING		The service pause is pending. 
//				SERVICE_PAUSED				The service is paused.    
//
//  RETURN VALUES:
//		ERROR_SUCCESS						The operation completed successfully.
//		ERROR_ACCESS_DENIED 				The specified service control manager 
//											database handle does not have access 
//											to the service.
//		ERROR_INVALID_HANDLE 				The specified handle is invalid.
//		ERROR_INVALID_NAME					The specified service name is invalid. 
//		ERROR_SERVICE_DOES_NOT_EXIST		The specified service does not exist.
//		ERROR_DEPENDENT_SERVICES_RUNNING	The service cannot be stopped 
//											because other running services are 
//											dependent on it. 
//		ERROR_INVALID_SERVICE_CONTROL		The requested control code is not valid, 
//											or it is unacceptable to the service. 
//		ERROR_SERVICE_CANNOT_ACCEPT_CTRL	The requested control code cannot be
//											sent to the service because the 
//											state of the service is SERVICE_STOPPED,
//											SERVICE_START_PENDING, or SERVICE_STOP_PENDING.
//		ERROR_SERVICE_NOT_ACTIVE			The service has not been started.
//		ERROR_SERVICE_REQUEST_TIMEOUT		The service did not respond to the start 
//											request in a timely fashion.
//		ERROR_DATABASE_DOES_NOT_EXIST		The specified database does not exist. 
//		ERROR_INVALID_PARAMETER				A specified parameter is invalid.
//
DWORD CheckService(char *servName, DWORD &status);


//
//  FUNCTION: BOOL CheckServices(char *ibuf, char *obuf, DWORD &nBytes)
//
//  PURPOSE: Check all the services specified in ibuf and report the status
//			 for each service in obuf
//
//  PARAMETERS:
//		ibuf - buffer received from client which contains all the services that are to
//			   be checked.  The buffer has the following format:
//				-----------------------------------------
//				|id=ID;key=KEY;serv1;serv2;  ;servN;;|
//				-----------------------------------------
//	
//		obuf - buffer to be sent back, conatins the services + the service status codes
//				---------------------------------------------------
//				|nbytes=N;serv1=STAT;serv2=STAT;  ;servN=STAT;;|
//				---------------------------------------------------
//		
//		nBytes - Pointer to a DWORD that contains the maximum size, in characters, 
//				 of obuf. This also the value added to the 'nbytes=' field of obuf
//				 to tell the client how much space is needed.
//				 If nbytes is too small, it will point to the number of bytes required 
//				 when the funtion returns, and obuf will not contain any data.
//    
//  RETURN VALUES:
//		TRUE -  everything OK, obuf is ready to be sent back.
//		FALSE - nBytes is too small, obuf conatins no valid data.
//				
//
BOOL  CheckServices(char *ibuf, char *obuf, DWORD &nBytes);


//
//  FUNCTION: DWORD GetBytesNeededForServiceCheckBuffer(char *ibuf)
//
//  PURPOSE: Compute number of bytes needed to build the buffer to 
//			 end back to the client.
//
//  PARAMETERS: 
//		ibuf - buffer received from client which contains all the services that are to
//			   be checked.
//    
//  RETURN VALUES:
//		Number of bytes needed
//				
//		
DWORD GetBytesNeededForServiceCheckBuffer(char *ibuf);


//
//  FUNCTION: void AddServStatToBuffer(char *buf, char *servName, int status)
//
//  PURPOSE: Add service name + status code to a buffer: SERVNAME=STATUS;		 
//
//  PARAMETERS: 
//		buf - buffer to which the service name + status code are added to 
//		servName - name of the service 
//		status	- current status code of the specified service
//	
//	RETURN VALUES:
//		none
//
VOID AddServStatToBuffer(char *buf, char *servName, int status);

#endif
