#ifndef _nsservH
#define _nsservH

#include <iostream.h>
#include <winsock.h>
#include <math.h>
#include <stdio.h>
#include <fstream.h>
#include <iomanip.h>
#include "cpuLoad.h"
#include "err.h"
#include "service.h"
#include "common.h"
#include "eventLog.h"
#include "netutil.h"
#include "checkService.h"

// file names
#define MSGFILE				"msgLog.txt"
#define INIFILE				"NSServ.ini"
#define ERRLOG				"errMainLog.txt"


//
//  FUNCTION: int AnalyzeUDPMessage(char *ibuf, char *msgBuf, char *passwd)
//
//  PURPOSE: Read incoming buffer from client, and reply according to
//			 the request type.			 
//
//  PARAMETERS:
//		ibuf - message (request) received from client
//		msgBuf - message (reply) to send back to client
//		passwd - password from passwordfile
//		
//  RETURN VALUE:
//		NOERR - OK, msgBuf ready to be sent back to client
//		ERR_AUTHENTICATION - the password was wrong
//		ERR_NO_SUCH_MESSAGE - the request is undefined
//
int AnalyzeUDPMessage(char *ibuf, char *msgBuf, char *passwd);


//
//  FUNCTION: int GetCPULoadInfo(char *ibuf, char *obuf)
//
//  PURPOSE: Read ibuf and bulid a buffer with cpu-load values
//			 corresponding to the parameters given in ibuf		 
//
//  PARAMETERS:
//		ibuf - buffer received from client
//		obuf - bufffer to send back to client
//		
//  RETURN VALUE:
//		NOERR -  OK, obuf ready to be sent back to client
//		ERR_CPULOAD_INTERVAL - error in interval values
//
//	COMMENTS:
//		ibuf has the following format:
//		----------------------------------------
//		|id=ID;key=KEY;i1=INTERVAL1;i2=INTERVAL2|
//		-----------------------------------------
//		id - message id
//		key - password
//		i1 - first interval (in minutes) to compute cpu load average for
//		i2 - second interval (in minutes) to compute cpu load average for
//
//		obuf has the following format:
//		-----------------------------------------------------
//		|id=ID;l1=LOAD1;l2=LOAD2;pname=PROCNAME;pload=PLOAD;|
//		-----------------------------------------------------
//		id - message id
//		l1 - cpu load average corresponding to to intreval i1 in ibuf
//		l2 - cpu load average corresponding to to intreval i2 in ibuf
//		pname - name of process with highest cpu load
//		pload - cpu load of process with highest cpu load
//
int GetCPULoadInfo(char *ibuf, char *obuf);


//
//  FUNCTION: int Round(double number)
//
//  PURPOSE: round a decimal value to an integer
//
//  PARAMETERS: 
//		number - decimal number to round off
//		
//  RETURN VALUE:
//		rounded number
//
int Round(double);


//
//  FUNCTION: int ReadInitData(char *passwd, int &port, char *installDir)
//
//  PURPOSE: Read initialation data from the INIFILE located in %SystemRoot%
//
//  PARAMETERS:
//		passwd - variable to hold the password read from the file
//		port - portnumber to use (both UDP and TCP connection)
//		installdir - tells us in which directory we are installed
//		
//  RETURN VALUE:
//		NOERR - reading INIFILE was success
//		ERR_NOINIFILE - the INIFIL was not found
//		ERR_FILEFORMAT - error in fileformat of INIFILE
//
int ReadInitData(char *passwd, int &port, char *installDir);

 
//
//  FUNCTION: int CreateUDPSocket(SOCKET &servSock, int port)
//
//  PURPOSE: Create a UDP socket which listens to the specified port		 
//
//  PARAMETERS:
//		servSock - socket descriptor which identifies the connection
//		port - portnumber to listen to
//		
//  RETURN VALUE:
//		NOERR - the socket was successfully created
//		ERR_SOCKETINIT - socket initiation error
//		ERR_BIND - error binding the socket
//
int CreateUDPSocket(SOCKET &servSock, int port);


//
//  FUNCTION: int ReadLine(char* str, int StrSize, HANDLE hFile)
//
//  PURPOSE: read a line from the the specfied file refrenced by hFile			 
//
//  PARAMETERS:
//		str - string to contain the line read from the file
//		StrSize - size of str
//		hFile - handle to file which we want to read from
//		
//  RETURN VALUE:
//		NOERR - operation OK
//		1 - error reading the file
//
int ReadLine(char* str, int StrSize, HANDLE hFile);


//
//  FUNCTION: BOOL CheckThread(HANDLE hCpuLoadThrd, DWORD &dwCpuLoadThrdId)
//
//  PURPOSE: Check that the cpu-thread is running, restart if it has terminated			 
//
//  PARAMETERS:
//		hCpuLoadThrd - handle to the cpu-thread
//		dwCpuLoadThrdId - cpu-thread identificator
//		
//  RETURN VALUE:
//		TRUE - cpu-thread is running
//		FALSE - cpu-thread has terminated and could not be restarted
//
BOOL CheckThread(HANDLE hCpuLoadThrd, DWORD &dwCpuLoadThrdId);


//
//  FUNCTION: Initialize(SOCKET &servSock, char* passwd)
//
//  PURPOSE: Initialize various stuff:
//				- read values from INIFILE
//				- add message-file to registry
//				- create UDP socket
//				- start the TCP-thread
//				- start the CPU-load thread
//
//  PARAMETERS:
//		servSock - socket which describes the UDP connection
//		passwd - password read from INIFILE
//		
//  RETURN VALUE:
//		TRUE - funtion returned successfully
//		FALSE - error
//
BOOL Initialize(SOCKET &servSock, char* passwd);


#endif